@@ -6,21 +6,19 @@ Basic wrapper for the Aladhan prayer times API.
 
 from collections import namedtuple
 
-VersionInfo = namedtuple(
-    "VersionInfo", "major minor micro releaselevel serial"
-)
+VersionInfo = namedtuple("VersionInfo", "major minor micro releaselevel serial")
 
 version_info = VersionInfo(
-    major=1, minor=1, micro=0, releaselevel="final", serial=0
+    major=1, minor=2, micro=0, releaselevel="final", serial=0
 )
 
 __title__ = "aladhan.py"
 __author__ = "HETHAT"
-__version__ = "1.1.0"
+__version__ = "1.2.0"
 
 
-from .client import Client
-from .base_types import *
 from . import methods
-from .methods import Method
+from .client import Client
+from .data_classes import *
 from .enums import *
+from .methods import Method
