@@ -1,31 +1,34 @@
-from .methods import all_methods, Method
-from .base_types import (
-    Timings,
-    Data,
-    TimingsDateArg,
-    Parameters,
+from typing import Awaitable as Aw
+from typing import Dict, List, Optional
+from typing import Union as Un
+
+from .data_classes import (
     CalendarDateArg,
-    Qibla,
-    Ism,
+    Data,
     Date,
+    Ism,
+    NextPrayerData,
+    Parameters,
+    Prayer,
+    Qibla,
+    Timings,
+    TimingsDateArg,
 )
 from .http import HTTPClient
+from .methods import Method, all_methods
+from .types import IMR, SDR, StatusR
 
-from typing import (
-    Awaitable,
-    Optional,
-    Union,
-    List,
-    Dict,
-)
-
-TimingsR = Union[Timings, Awaitable[Timings]]
-_Calendar = Union[List[Timings], Dict[str, Timings]]
-CalendarR = Union[_Calendar, Awaitable[_Calendar]]
-QiblaR = Union[Qibla, Awaitable[Qibla]]
-AsmaR = Union[List[Ism], Awaitable[List[Ism]]]
-DateR = Union[Date, Awaitable[Date]]
-LDateR = Union[List[Date], Awaitable[List[Date]]]
+TimingsR = Un[Timings, Aw[Timings]]
+_Calendar = Un[List[Timings], Dict[str, Timings]]
+CalendarR = Un[_Calendar, Aw[_Calendar]]
+QiblaR = Un[Qibla, Aw[Qibla]]
+AsmaR = Un[List[Ism], Aw[List[Ism]]]
+DateR = Un[Date, Aw[Date]]
+LDateR = Un[List[Date], Aw[List[Date]]]
+PrayerR = Un[Prayer, Aw[Prayer]]
+IntR = Un[int, Aw[int]]
+StrR = Un[str, Aw[str]]
+ListR = Un[list, Aw[list]]
 
 __all__ = ("Client",)
 
@@ -35,49 +38,64 @@ class Client:
     Al-adhan API client.
 
     Set to synchronous usage by default, set is_async to True
-        if asynchronous usage wanted.
+    if asynchronous usage wanted.
 
-    Synchronous example
+    Auto handles rate limits by default, set auto_manage_rate to False if
+    otherwise is wanted. *New in v1.2.0*
 
-    .. code:: py
+    Example
 
-        import aladhan
+    .. tab:: Synchronous
 
-        client = aladhan.Client()
-        times = client.get_timings_by_address("New York")
-        print(times)
+        .. code:: py
 
-    Asynchronous example
+            import aladhan
 
-    .. code:: py
-
-        import aladhan, asyncio
-
-        async def main():
-            # --- manual closing session
-            client = aladhan.Client(is_async=True)
-            times = await client.get_timings_by_address("New York")
+            client = aladhan.Client()
+            times = client.get_timings_by_address("New York")
             print(times)
-            await client.close()
 
-            # --- using context
-            async with aladhan.Client(is_async=True) as client:
+    .. tab:: Asynchronous
+
+        .. code:: py
+
+            import aladhan, asyncio
+
+            async def main():
+                # --- manual closing session
+                client = aladhan.Client(is_async=True)
                 times = await client.get_timings_by_address("New York")
                 print(times)
+                await client.close()
 
-        loop = asyncio.get_event_loop()
-        loop.run_until_complete(main())
+                # --- using context
+                async with aladhan.Client(is_async=True) as client:
+                    times = await client.get_timings_by_address("New York")
+                    print(times)
+
+            loop = asyncio.get_event_loop()
+            loop.run_until_complete(main())
 
     .. note::
-        For Asynchronous usage you need to initialize
-            this class in a |coroutine_link|_.
+        For Asynchronous usage you need to initialize this class in
+        a |coroutine_link|_.
+
+    Parameters
+    ----------
+        is_async: :class:`bool`
+            Whether to be used asynchronously or not.
+
+        auto_manage_rate: :class:`bool`
+            Whether to handle rate limits automatically or not.
     """
 
     __slots__ = "converter", "http"
 
-    def __init__(self, is_async: bool = False):
+    def __init__(self, is_async: bool = False, auto_manage_rate: bool = True):
         self.converter = is_async and _AsyncConverter or _SyncConverter
-        self.http = HTTPClient(is_async=is_async)
+        self.http = HTTPClient(
+            is_async=is_async, auto_manage_rate=auto_manage_rate
+        )
 
     def close(self):
         """Closes the connection."""
@@ -109,10 +127,52 @@ class Client:
     async def __aexit__(self, *_):
         await self.close()
 
+    def get_next_prayer_by_address(
+        self,
+        address: str,
+        date: Optional[TimingsDateArg] = None,
+        params: Optional[Parameters] = None,
+    ) -> PrayerR:
+        """
+        Get next upcoming prayer from address.
+
+        Parameters
+        ----------
+            address: :class:`str`
+                An address string.
+                Example: "London, United Kingdom"
+
+            date: Optional[:class:`TimingsDateArg`]
+                Date for the prayer times.
+                Default: Current date.
+
+            params: Optional[:class:`Parameters`]
+                Default: ``Parameters()``
+
+        Returns
+        -------
+            :class:`Prayer`
+                Prayer obj from the API response.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+                Invalid parameter was passed.
+
+        *New in v1.2.0*
+        """
+        date = date and date.date or ""
+        params = (params or Parameters()).as_dict
+        params.update(dict(address=address))
+        return self.converter.to_prayer(
+            self,
+            self.http.get_next_prayer_by_address(date, params),
+        )
+
     def get_timings(
         self,
-        longitude: Union[int, float],
-        latitude: Union[int, float],
+        longitude: Un[int, float],
+        latitude: Un[int, float],
         date: Optional[TimingsDateArg] = None,
         params: Optional[Parameters] = None,
     ) -> TimingsR:
@@ -120,7 +180,7 @@ class Client:
         Get prayer times from coordinates (longitude, latitude).
 
         Parameters
-        -----------
+        ----------
             longitude: :class:`int` or :class:`float`
                 Longitude coordinate of the location.
 
@@ -144,7 +204,7 @@ class Client:
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
         """
-        date = (date or TimingsDateArg()).date
+        date = date and date.date or ""
         params = (params or Parameters()).as_dict
         params.update(dict(longitude=str(longitude), latitude=str(latitude)))
         return self.converter.to_timings(
@@ -161,7 +221,7 @@ class Client:
         Get prayer times from address.
 
         Parameters
-        -----------
+        ----------
             address: :class:`str`
                 An address string.
                 Example: "London, United Kingdom"
@@ -183,7 +243,7 @@ class Client:
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
         """
-        date = (date or TimingsDateArg()).date
+        date = date and date.date or ""
         params = (params or Parameters()).as_dict
         params.update(dict(address=address))
         return self.converter.to_timings(
@@ -202,7 +262,7 @@ class Client:
         Get prayer times from city, country and state.
 
         Parameters
-        -----------
+        ----------
             city: :class:`str`
                 The city name.
                 Example: "London"
@@ -232,7 +292,7 @@ class Client:
             :exc:`~aladhan.exceptions.BadRequest`
                 Invalid parameter was passed.
         """
-        date = (date or TimingsDateArg()).date
+        date = date and date.date or ""
         params = (params or Parameters()).as_dict
         params.update(dict(city=city, country=country, state=state))
         if state is None:
@@ -243,8 +303,8 @@ class Client:
 
     def get_calendar(
         self,
-        longitude: Union[int, float],
-        latitude: Union[int, float],
+        longitude: Un[int, float],
+        latitude: Un[int, float],
         date: CalendarDateArg,
         params: Optional[Parameters] = None,
     ) -> CalendarR:
@@ -253,7 +313,7 @@ class Client:
         coordinates (longitude, latitudes).
 
         Parameters
-        -----------
+        ----------
             longitude: :class:`int` or :class:`float`
                 Longitude coordinate of the location.
 
@@ -296,7 +356,7 @@ class Client:
             from address.
 
         Parameters
-        -----------
+        ----------
             address: :class:`str`
                 An address string.
                 Example: "London, United Kingdom"
@@ -393,7 +453,7 @@ class Client:
         return all_methods
 
     def get_qibla(
-        self, longitude: Union[int, float], latitude: Union[int, float]
+        self, longitude: Un[int, float], latitude: Un[int, float]
     ) -> QiblaR:
         """
         Get the Qibla direction from a pair of coordinates.
@@ -418,8 +478,8 @@ class Client:
 
         *New in v0.1.3*
         """
-        return self.converter.to_qibla(
-            self.http.get_qibla((latitude, longitude))
+        return self.converter.to_obj_kwa(
+            self.http.get_qibla(latitude, longitude), Qibla
         )
 
     def get_asma(self, *n: int) -> AsmaR:
@@ -445,8 +505,8 @@ class Client:
         """
 
         assert n, "No arguments was passed."
-        return self.converter.to_asma(
-            self.http.get_asma(",".join(map(str, n)))
+        return self.converter.to_list_of_obj(
+            self.http.get_asma(",".join(map(str, n))), Ism
         )
 
     def get_all_asma(self) -> AsmaR:
@@ -490,9 +550,11 @@ class Client:
 
         *New in v1.1.0*
         """
-        params = dict(date=date.date, adjustment=adjustment)
-        return self.converter.to_date(
-            self.http.get_hijri_from_gregorian(params)
+        return self.converter.to_obj_kwa(
+            self.http.get_hijri_from_gregorian(
+                date=date.date, adjustment=adjustment
+            ),
+            Date,
         )
 
     def get_gregorian_from_hijri(
@@ -523,9 +585,11 @@ class Client:
 
         *New in v1.1.0*
         """
-        params = dict(date=date.date, adjustment=adjustment)
-        return self.converter.to_date(
-            self.http.get_gregorian_from_hijri(params)
+        return self.converter.to_obj_kwa(
+            self.http.get_gregorian_from_hijri(
+                date=date.date, adjustment=adjustment
+            ),
+            Date,
         )
 
     def get_hijri_calendar_from_gregorian(
@@ -553,10 +617,11 @@ class Client:
 
         *New in v1.1.0*
         """
-        return self.converter.to_calendar(
+        return self.converter.to_list_of_obj(
             self.http.get_hijri_calendar_from_gregorian(
-                (month, year, adjustment)
-            )
+                month, year, adjustment
+            ),
+            Date,
         )
 
     def get_gregorian_calendar_from_hijri(
@@ -584,10 +649,11 @@ class Client:
 
         *New in v1.1.0*
         """
-        return self.converter.to_calendar(
+        return self.converter.to_list_of_obj(
             self.http.get_gregorian_calendar_from_hijri(
-                (month, year, adjustment)
-            )
+                month, year, adjustment
+            ),
+            Date,
         )
 
     def get_islamic_year_from_gregorian_for_ramadan(self, year: int) -> int:
@@ -609,82 +675,280 @@ class Client:
             :exc:`~aladhan.exceptions.BadRequest`
                 Unable to compute year.
         """
-        return self.converter.to_int(
-            self.http.get_islamic_year_from_gregorian_for_ramadan(year)
+        return self.converter.to_obj_a(
+            self.http.get_islamic_year_from_gregorian_for_ramadan(year), int
         )
 
+    def get_current_time(self, zone: str) -> StrR:
+        """
+        Parameters
+        ----------
+            zone: :class:`str`
+                Timezone string. ex: `"Europe/London"`
+
+        Returns
+        -------
+            :class:`str`
+                The current time for the specified time zone.
+                ex: `"13:56"`
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+                Invalid timezone.
+        """
+        return self.http.get_current_time(zone=zone)
+
+    def get_current_date(self, zone: str) -> StrR:
+        """
+        Parameters
+        ----------
+            zone: :class:`str`
+                Timezone string. ex: `"Europe/London"`
+
+        Returns
+        -------
+            :class:`str`
+                The current Date for the specified time zone in DD-MM-YYYY.
+                ex: `"23-02-2021"`
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+                Invalid timezone.
+        """
+        return self.http.get_current_date(zone=zone)
+
+    def get_current_timestamp(self, zone: str) -> IntR:
+        """
+        Parameters
+        ----------
+            zone: :class:`str`
+                Timezone string. ex: `"Europe/London"`
+
+        Returns
+        -------
+            :class:`int`
+                The current UNIX timestamp for the specified time zone.
+                ex: `1503495668`
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+                Invalid timezone.
+        """
+        return self.converter.to_obj_a(
+            self.http.get_current_timestamp(zone=zone), int
+        )
+
+    def get_current_islamic_year(self, adjustment: int = 0) -> IntR:
+        """
+        Parameters
+        ----------
+            adjustment: :class:`int`
+                Number of days to adjust hijri date.
+
+        Returns
+        -------
+            :class:`int`
+                The current islamic year.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+               Unable to compute year.
+        """
+        return self.converter.to_obj_a(
+            self.http.get_current_islamic_year(adjustment=adjustment), int
+        )
+
+    def get_current_islamic_month(self, adjustment: int = 0) -> IntR:
+        """
+        Parameters
+        ----------
+            adjustment: :class:`int`
+                Number of days to adjust hijri date.
+
+        Returns
+        -------
+            :class:`int`
+                The current islamic month.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+               Unable to compute month.
+        """
+        return self.converter.to_obj_a(
+            self.http.get_current_islamic_month(adjustment=adjustment), int
+        )
+
+    def get_next_hijri_holiday(self, adjustment: int = 0) -> DateR:
+        """
+        Parameters
+        ----------
+            adjustment: :class:`int`
+                Number of days to adjust hijri date.
+
+        Returns
+        -------
+            :class:`~aladhan.Date`
+                The Next upcoming hijri holiday.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+               Unable to compute next holiday.
+        """
+        return self.converter.to_obj_kwa(
+            self.http.get_next_hijri_holiday(adjustment=adjustment), Date
+        )
+
+    def get_hijri_holidays(self, day: int, month: int) -> ListR:
+        """
+        Parameters
+        ----------
+            day: :class:`int`
+                Hijri day.
+            month: :class:`int`
+                Hijri month.
+
+        Returns
+        -------
+            :class:`list` of :class:`str`
+                All day's holidays, can be empty list.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+               Invalid day or month.
+        """
+        return self.http.get_hijri_holidays(day, month)
+
+    def get_islamic_holidays(self, year: int, adjustment: int = 0) -> LDateR:
+        """
+        Parameters
+        ----------
+            year: :class:`int`
+                Hijri year.
+            adjustment: :class:`int`
+                Number of days to adjust hijri date.
+
+        Returns
+        -------
+            :class:`list` of :class:`~aladhan.Date`
+                All year's holidays, 19 in total.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+               Something went wrong.
+        """
+        return self.converter.to_list_of_obj(
+            self.http.get_hijri_holidays(year, adjustment), Date
+        )
+
+    def get_status(self) -> StatusR:
+        """
+        Returns
+        -------
+            :class:`dict`
+                Api's status.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.InternalServerError`
+               Status Check Failed.
+        """
+        return self.http.get_status()
+
+    def get_special_days(self) -> SDR:
+        """
+        Get all islamic special days (holidays).
+
+        Returns
+        -------
+            :class:`list` of :class:`dict`
+                A list of all special days with ``day,month,name`` keys.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+               Something went wrong.
+        """
+        return self.http.get_special_days()
+
+    def get_islamic_months(self) -> IMR:
+        """
+        Get all 12 islamic months.
+
+        Returns
+        -------
+            :class:`dict` of :class:`str` and :class:`dict`
+                A dict of '1' to '12' as keys and dicts of ``number,en,ar`` keys.
+
+        Raises
+        ------
+            :exc:`~aladhan.exceptions.BadRequest`
+               Something went wrong.
+        """
+        return self.http.get_islamic_months()
+
+
+def _decide_timings(client, data):
+
+    if isinstance(data, list):  # it is a month calendar
+        return [Data(**day, client=client).timings for day in data]
+    # it is a dict
+    if "1" in data:  # it is a year calendar
+        return {
+            month: [Data(**day, client=client).timings for day in days]
+            for month, days in data.items()
+        }
+
+    # it is just a day timings
+    return Data(**data, client=client).timings
+
 
 class _SyncConverter:
+    @staticmethod
+    def to_prayer(client, o):
+        return NextPrayerData(client=client, **o).prayer
+
     @staticmethod
     def to_timings(client, o):
-
-        data = o
-        if isinstance(data, list):  # it is a month calendar
-            return [Data(**day, client=client).timings for day in data]
-        # it is a dict
-        if "1" in data:  # it is a year calendar
-            return {
-                month: [Data(**day, client=client).timings for day in days]
-                for month, days in data.items()
-            }
-
-        # it is just a day timings
-        return Data(**data, client=client).timings
+        return _decide_timings(client, o)
 
     @staticmethod
-    def to_qibla(o):
-        return Qibla(**o)
+    def to_obj_a(o, obj):
+        return obj(o)
 
     @staticmethod
-    def to_asma(o):
-        return [Ism(**d) for d in o]
+    def to_obj_kwa(o, obj):
+        return obj(**o)
 
     @staticmethod
-    def to_date(o):
-        return Date(**o)
-
-    @staticmethod
-    def to_calendar(o):
-        return [Date(**d) for d in o]
-
-    @staticmethod
-    def to_int(o):
-        return int(o)
+    def to_list_of_obj(o, obj):
+        return [obj(**d) for d in o]
 
 
 class _AsyncConverter:
+    @staticmethod
+    async def to_prayer(client, o):
+        return NextPrayerData(client=client, **(await o)).prayer
+
     @staticmethod
     async def to_timings(client, o):
-
-        data = await o
-        if isinstance(data, list):  # it is a month calendar
-            return [Data(**day, client=client).timings for day in data]
-        # it is a dict
-        if "1" in data:  # it is a year calendar
-            return {
-                month: [Data(**day, client=client).timings for day in days]
-                for month, days in data.items()
-            }
-
-        # it is just a day timings
-        return Data(**data, client=client).timings
+        return _decide_timings(client, await o)
 
     @staticmethod
-    async def to_qibla(o):
-        return Qibla(**(await o))
+    async def to_obj_a(o, obj):
+        return obj(await o)
 
     @staticmethod
-    async def to_asma(o):
-        return [Ism(**d) for d in await o]
+    async def to_obj_kwa(o, obj):
+        return obj(**(await o))
 
     @staticmethod
-    async def to_date(o):
-        return Date(**(await o))
-
-    @staticmethod
-    async def to_calendar(o):
-        return [Date(**d) for d in await o]
-
-    @staticmethod
-    async def to_int(o):
-        return int(await o)
+    async def to_list_of_obj(o, obj):
+        return [obj(**d) for d in await o]
