@@ -2,6 +2,7 @@ __all__ = (
     "AladhanException",
     "HTTPException",
     "BadRequest",
+    "TooManyRequests",
     "InternalServerError",
     "InvalidArgument",
     "InvalidMethod",
@@ -11,6 +12,7 @@ __all__ = (
     "InvalidTimezone",
     "InvalidLatAdjMethod",
     "InvalidAdjustment",
+    "InvalidShafaq",
 )
 
 
@@ -51,19 +53,30 @@ class HTTPException(AladhanException):
     def __init__(self, response, message: str = None):
         self.response = response
         self.code = response.get("code", 0)
-        super().__init__(message or response.get("data"))
+        super().__init__(message or response.get("message"))
 
     @classmethod
     def from_res(cls, res):
-        return {400: BadRequest, 500: InternalServerError}.get(
-            res.get("code"), cls
-        )(res)
+        return {
+            400: BadRequest,
+            429: TooManyRequests,
+            500: InternalServerError,
+        }.get(res.get("code"), cls)(res)
 
 
 class BadRequest(HTTPException):
     """Exception that’s thrown for when status code 400 occurs."""
 
 
+class TooManyRequests(HTTPException):
+    """Exception that’s thrown for when status code 429 occurs.
+
+    .. note::
+        Current API's rate limit is 14 requests/s.
+
+    *New in v1.2.0*"""
+
+
 class InternalServerError(HTTPException):
     """Exception that’s thrown for when status code 500 occurs."""
 
@@ -100,3 +113,7 @@ class InvalidLatAdjMethod(InvalidArgument):
 
 class InvalidAdjustment(InvalidArgument):
     """Exception that’s thrown when adjustment argument is invalid."""
+
+
+class InvalidShafaq(InvalidArgument):
+    """Exception that’s thrown when shafaq argument is invalid."""
